'use strict';

const util = require('util');
const log = require('@ntnu/logger')()('config-loader');

class Config {

  constructor() {
    Object.defineProperty(this, 'secrets', { value: [] });
    Object.defineProperty(this, 'fromEnv', { value: [] });
  }

  [util.inspect.custom](depth, options) {
    return Object.keys(this).reduce((acc, key) => {
      var value = this[key];
      if (value && this.secrets.includes(key)) {
        value = options.stylize('******', 'red');
      }
      if (this.fromEnv.includes(key)) {
          key = key + options.stylize('<ENV>', 'grey');
      }
      acc[key] = value;
      return acc;
    }, {});
  }
}

function join(...args) {
  return args.filter((x) => x).join('_');
}

function mapObject(prefix, obj, keypath) {
  return Object.keys(obj).reduce((acc, key) => {
    const value = obj[key];
    if (!Array.isArray(value) && typeof value === 'object') {
      acc[key] = mapObject(prefix, value, join(keypath, key));
      return acc;
    }

    const newKey = key.replace(/^secret_/, '');
    if (newKey !== key) {
      acc.secrets.push(newKey);
    }

    const envVarName = join(keypath, newKey).toUpperCase();
    const prefixedEnvVarName = join(prefix, envVarName);
    let envVar = process.env[prefixedEnvVarName];
    if (envVar === undefined) {
      envVar = process.env[envVarName];
    }

    if (envVar !== undefined) {
      acc.fromEnv.push(newKey);
      if (Array.isArray(value)) {
        acc[newKey] = envVar.split(',');
      } else if (typeof value === 'boolean') {
        acc[newKey] = envVar.toLowerCase() === 'true';
      } else if (typeof value === 'number') {
        acc[newKey] = parseInt(envVar);
      } else {
        acc[newKey] = envVar;
      }
    } else {
      acc[newKey] = value;
    }

    return acc;
  }, new Config());
}


module.exports = function (prefix, config) {
  var res = mapObject(prefix, config);
  log.info('Config', util.inspect(res, {depth: 10, compact: false}));
  return res;
};
